/****************************************************************
 *                 LFM
 *             NVCom-02T-1508PL8T                       *
 *      Демонстрирует работу с микросхемой 1508PL8T по SPI:     *
 *      Режим синтеза гармонического сигнала.
 *      Длительность стадии - 1мкс, в тактах - 10^(-6) * 800*10^6 = 800 тактов
 *      Разница частот 25МГц (до 50МГц)
 *      Приращение частоты за такт в герцах dFq_mhz = dF/dT =25*10*^6/800 = (25/8)*10^4 =3,125*10^4 Гц
 *
 *      Приращение частоты за такт (значение для регистров ПЛ8Т):
 *      dFq = (dFq_mhz/Fclk) *2^48 = 3,125*10^4/(800*10^6) *2^48 = (3,125/80000) *2^48= 10995116277,76 = 0x100_0000_0000
 *      Выводы разъемов X10 и XP8                               *
 *      модулей 1508ПЛ8Т(Радиокомп) и NVCom-02TEM-3U            *
 *      должный быть соединены следующим образом:
 *      X10         XP8
 *      SCK         12 (LCLKn)
 *      SDI(MOSI)   4  (LDAT[3])
 *      SCS         2  SS_0(LDAT[1])
 *      S23(MISO)   3  (LDAT[2])
 *      GND         14 (GND)
 *                                                              *
 ***************************************************************/



// файл со ссылками на макроопределения регистров, в том числе нужных нам CLK_EN, DIR_MFBSP1, GPIO_DR1
#include "multicore/nvcom02t.h"
// файл со ссылками адресов регистров микросхемы 1508ПЛ8Т и команд
#include "dds.h"

// установка длительности паузы в тактах частоты CPU, 1 секунда
#define TIMEOUT 100000000

// частота работы SPI в Гц
unsigned int SPI_FRQ = 6000000;
// частота работы процессора в герцах
#define FREQ 240000000

unsigned int SYNT1_read(unsigned int addr){

	MFBSP3.TX.data = COMMAND_SETAFT|(addr & 0xFFFF);

	while (!(MFBSP3.TSR.bits.TBE));
	 while (!(MFBSP3.CSR.data & 1<<4));     //ожидание получения новых данных
		return (MFBSP3.RX.data & 0xFFFF); // приходят данные от предыдущей команды, это не ответ на команду чтения
}


void SYNT1_write(unsigned int addr, unsigned int val){ // Функция записи в адресное пространство DDS

	MFBSP3.TX.data = COMMAND_SETA|(addr & 0xFFFF);
	while (!(MFBSP3.TSR.bits.TBE));

	MFBSP3.TX.data = COMMAND_WR|(val & 0xFFFF);
	while (!(MFBSP3.TSR.bits.TBE));

}

void SYNT1_RESET(){
	SYNT1_write(0x0,0x78); //RESET
}

void SPI_Init() {
	        MFBSP3.CSR.bits.SPI_I2S_EN = 1; // включен режим I2S/SPI Появляется 201 в TX

	        MFBSP3.DIR.bits.TD_DIR = 1;    // MOSI- выход. MISO - вход по умолчанию
	        MFBSP3.DIR.bits.TCS_DIR = 1;   // SS[0] - выход
	        MFBSP3.DIR.bits.TCLK_DIR =1;   // TSCK - выход
	        MFBSP3.DIR.bits.RCLK_DIR =0;   // tiu RSCK - формируется приемником (то есть, нами же)
	        MFBSP3.DIR.bits.RD_DIR = 0;    // tiu MISO - вход по умолчанию
// настройка передатчика
	        MFBSP3.TCTR.bits.SS_0 = 1;      // Используется SS[0] в качестве CS
	        MFBSP3.TCTR.bits.TWORDLEN = 23; // длина слова - 24 разряда
	        MFBSP3.TCTR.bits.TMBF = 1;      // старшим битом вперед
	        MFBSP3.TCTR.bits.TWORDCNT = 0; // одно слово во фрейме – чтобы сигнал SS[0]
						// переключался в единицу после передачи каждого слова
	        MFBSP3.TCTR.bits.TNEG = 1;      // TNEG=1, TDEL=1. При данном сочетании выдача данных
	        MFBSP3.TCTR.bits.TDEL = 1;      // происходит по заднему фронту TSCK
		                                                  // исходное состояние TSCK перед началом передачи - высокое.
	        MFBSP3.TCTR.bits.TMODE = 1;     // режим SPI (TMODE=0 – режим I2S)
	        MFBSP3.TCTR.bits.TPACK = 0;

	        MFBSP3.TCTR_RATE.data = (FREQ/(2*SPI_FRQ)) - 1; // частота работы SPI

// настройка приемника
	        MFBSP3.RCTR.bits.RWORDLEN = 23;// длина принимаемого слова - 24 разряда
	        MFBSP3.RCTR.bits.RMBF = 1;	   // старшим битом вперед
            MFBSP3.RCTR.bits.RWORDCNT=0; // 1 слово во фрейме

// RSCK и SS[0] - берутся от передатчика
	        MFBSP3.RCTR.bits.RCS_CP = 1;
	        MFBSP3.RCTR.bits.RCLK_CP = 1;

	        MFBSP3.RCTR.bits.RMODE = 1;    // режим SPI



	        MFBSP3.RCTR.bits.RNEG = 1;       // RNEG=1, RDEL=1. При данном сочетании захват данных при
	        MFBSP3.RCTR.bits.RDEL = 1;       // приеме данных происходит по переднему фронту RSCK
	        		        		                                         // а исходное состояние RSCK перед началом приема - высокое.
	        		        		                                         // RSCK и SS[0] - берутся от передатчика

	        MFBSP3.RCTR.bits.REN = 1;	   // приемник включен
	        MFBSP3.RSTART.data = 1;
	        MFBSP3.TSTART.data = 1;
	        MFBSP3.TCTR.bits.TEN = 1;       // TEN - передатчик включен


	                                       // Такой порядок включения рекомендован разработчиком
}

unsigned int GetCP0_Count() {
                         unsigned int result;
                         asm volatile ("mfc0 %0, $9" :"=r"(result));
                         return result;
}

void SetCP0_Count(unsigned int value) {
                 asm volatile ("mtc0 %0, $9" ::"r"(value));
}

int main() {

	unsigned int rcv_array[24];
	unsigned int i;
	SPI_Init();

	SYNT1_write(SEL_REG,0x0000); //по умолчанию для первого канала выбран профиль 0
	SYNT1_write(CTR,0x1000);
	SYNT1_write(SYNC,0x0000);

	SYNT1_write(TC_L,0x0000);
	SYNT1_write(TC_H,0x0000);
	SYNT1_write(CH1_TSW,0x0000);

    // Необходимо задать значения профилей с 0-го по 4-ый, так как
    //при отключенном режиме корреции параметры фазы, амплитуды и постоянного смещения
	// синтезируемого сигнала берутся из профиля с номерами 1, 2, 3, 0
	// для стадий 1-4 соответственно.
	SYNT1_write(CH1_P0,0x0000);
	SYNT1_write(CH1_Mul0,0x7fff);
	SYNT1_write(CH1_Offset0,0x0000);

	SYNT1_write(CH1_P1,0x0000);
	SYNT1_write(CH1_Mul1,0x7fff);
	SYNT1_write(CH1_Offset1,0x0000);

	// Записываем во все профили
	SYNT1_write(CH1_dPh_all_L,0x0000);
	SYNT1_write(CH1_dPh_all_M,0x0000);
	SYNT1_write(CH1_dPh_all_H,0x0000);
	SYNT1_write(CH1_P_all,0x0000);
	SYNT1_write(CH1_Mul_all,0x7fff);
	SYNT1_write(CH1_Offset_all,0x0000);



    // Задается начальная частота стадии 1
	SYNT1_write(CH1_LS_F1_L,0x0000);
	SYNT1_write(CH1_LS_F1_M,0x0000);
	SYNT1_write(CH1_LS_F1_H,0x0800);

	// Задается начальная частота стадии 3
    SYNT1_write(CH1_LS_F2_L,0x0000);
	SYNT1_write(CH1_LS_F2_M,0x0000);
	SYNT1_write(CH1_LS_F2_H,0x0800);

	// Задается приращение фазы (0)
	SYNT1_write(CH1_dPh0_L,0x0000);
	SYNT1_write(CH1_dPh0_M,0x0000);
	SYNT1_write(CH1_dPh0_H,0x0000);

    // Задается приращение частоты в 1-ой и 2-ой стадиях ЛЧМ
	SYNT1_write(CH1_LS_dFq1_L,0xA3D7);
	SYNT1_write(CH1_LS_dFq1_M,0x3D70);
	SYNT1_write(CH1_LS_dFq1_H,0x0000A); // 0x000A_3D70_A3D7

	SYNT1_write(CH1_LS_dFq2_L,0xA3D7);
	SYNT1_write(CH1_LS_dFq2_M,0x3D70);
	SYNT1_write(CH1_LS_dFq2_H,0x000A);

	// Задается длительность каждой стадии ЛЧМ
	SYNT1_write(CH1_LS_TPH1_L,0x00C8); // 1 мкс: 1*10^(-6)*800*10^ 6 /4 = 800/4 = 200 0xC8 в результате 1мкс
	SYNT1_write(CH1_LS_TPH1_M,0x0000);
	SYNT1_write(CH1_LS_TPH1_H,0x0000);

	SYNT1_write(CH1_LS_TPH2_L,0x00C8);
	SYNT1_write(CH1_LS_TPH2_M,0x0000);
	SYNT1_write(CH1_LS_TPH2_H,0x0000);

	SYNT1_write(CH1_LS_TPH3_L,0x00C8);
	SYNT1_write(CH1_LS_TPH3_M,0x0000);
	SYNT1_write(CH1_LS_TPH3_H,0x0000);

	SYNT1_write(CH1_LS_TPH4_L,0x00C8);
	SYNT1_write(CH1_LS_TPH4_M,0x0000);
	SYNT1_write(CH1_LS_TPH4_H,0x0000);

	// Включение частоты
	SYNT1_write(CH1_LS_CTR,0xBC70);
	SYNT1_write(CLR,0x0010);
	while (1);

}
